/*
 * This file is part of fdnpkg
 * Copyright (C) Mateusz Viste 2012
 */

#include <ctype.h>  /* tolower() */
#include <stdio.h>
#include <stdlib.h> /* system() */
#include <string.h> /* strcpy() */
#include "htget.h"
#include "fileexst.h"
#include "pkginst.h"
#include "libunzip.h"  /* callunzip(), zip_listfiles()... */


static void strtolower(char *mystring) {
  int x;
  for (x = 0; mystring[x] != 0; x++) mystring[x] = tolower(mystring[x]);
}


void pkginstall(struct pkgdb *pkgdb, char *pkgname, char *dosdir, char *tempdir, char *localfile) {
  char fname[512];
  char shellcmd[512];
  char zipfile[512];
  char *instrepo;
  struct pkgdb *pkgnode, *lastnode;
  struct pkgrepo *pkgrepo;
  int repoid;
  struct ziplist *ziplinkedlist, *curzipnode;
  FILE *fd;

  strtolower(pkgname); /* convert pkgname to lower case, because the http repo is probably case sensitive! */
  
  if (localfile != NULL) strcpy(zipfile, localfile); /* if it's a local file, then skip all network stuff */

  /* check if not already installed, if already here, print a message "you need to remove it first" */
  sprintf(fname, "%s\\packages\\%s.lst", dosdir, pkgname);
  if (fileexists(fname) != 0) { /* file exists -> package already installed */
    printf("Package %s is already installed! Remove it first if you need to upgrade.\r\n", pkgname);
    return;
  }

  if (zipfile[0] == 0) {
    /* look into the db to find the package */
    pkgnode = findpkg(pkgdb, pkgname, &lastnode);
    if (pkgnode == NULL) { /* no such package found in repositories */
      printf("No package '%s' found in online repositories.\r\n", pkgname);
      return;
    }

    /* if found - check the list of repositories */
    if (pkgnode->repolist == NULL) {
      printf("Package '%s' is not available in repositories.\r\n", pkgname);
      return;
    }

    if (pkgnode->repolist->nextrepo != NULL) { /* available from more than 1 repo.. */
        char userchoicestr[8];
        int userchoice;
        struct pkgrepo *xrepo;
        printf("\r\n%s is available from several repositories. Choose which one to use:\r\n", pkgname);
        repoid = 1;
        for (xrepo = pkgnode->repolist; xrepo != NULL; xrepo = xrepo->nextrepo) {
          printf(" ");
          printf("%d) %s %s (from %s)\r\n", repoid, pkgnode->name, xrepo->version, xrepo->repo);
          repoid += 1;
        }
        for (;;) {
          printf("Your choice: ");
          fgets(userchoicestr, 6, stdin);
          userchoice = atoi(userchoicestr);
          if ((userchoice < 1) || (userchoice >= repoid)) {
              printf("Invalid choice!\r\n");
            } else {
              break;
          }
        }
        repoid = userchoice;
      } else { /* available only from one repository - get it there */
        repoid = 1;
    }
    pkgrepo = pkgnode->repolist;
    for (; repoid > 1; repoid--) pkgrepo = pkgrepo->nextrepo;
    instrepo = pkgrepo->repo;

    /* download the package from repoid into the temp directory */
    sprintf(fname, "%s%s.zip", instrepo, pkgname);
    sprintf(zipfile, "%s\\fdnpkg.tmp", tempdir);
    printf("Downloading package %s...\r\n", fname);
    if (htget(fname, NULL, NULL, 0, zipfile) != 0) {
      printf("Error downloading package. Aborted.\r\n");
      return;
    }
  } /* if (zipfile[0] == 0) */

  /* list content of the ZIP file and put it into pkgname.lst - used also to check the validity of the zip file */
  ziplinkedlist = zip_listfiles(zipfile);
  if (ziplinkedlist == NULL) {
    puts("Error: Invalid zip archive! Package not installed.");
    return;
  }
  /* open the lst file */
  sprintf(shellcmd, "%s\\packages\\%s.lst", dosdir, pkgname);
  fd = fopen(shellcmd, "w");
  if (fd == NULL) {
    printf("Error: Could not create %s!\r\n", shellcmd);
    return;
  }
  /* write list of files in zip into the lst */
  for (curzipnode = ziplinkedlist; curzipnode != NULL; curzipnode = curzipnode->nextfile) fprintf(fd, "%s\r\n", curzipnode->filename);
  /* free the ziplist and close file descriptor */
  zip_freelist(&ziplinkedlist);
  fclose(fd);

  /* unzip files into %DOSDIR% (via the unzip -d directory switch) */
  sprintf(shellcmd, "unzip %s -d %s", zipfile, dosdir);
  printf("-> %s\r\n", shellcmd);
  /* system(shellcmd); */
  callunzip(shellcmd);
  printf("Package %s installed.\r\n", pkgname);
}
